import json
from datetime import date 
from datetime import datetime 
import array

class MatlabDataEncoder(json.JSONEncoder): 
    
    def default(self, field): 
     
        if isinstance(field, datetime): 
            return field.strftime('%Y-%m-%d %H:%M:%S') 
        elif isinstance(field, date): 
            return field.strftime('%Y-%m-%d') 
        elif isinstance(field,array.array) :
            return  field.tolist()
        elif isinstance(field,memoryview):
            return  field.tolist()
        else: 
            return json.JSONEncoder.default(self, field)  

class DateTimeEncode(json.JSONEncoder): 
    def default(self, field):
        if isinstance(field, datetime):
            return field.strftime('%Y-%m-%d %H:%M:%S') 
        elif isinstance(field, date):
            return field.strftime('%Y-%m-%d')
        else:
            return json.JSONEncoder.default(self, field)


import threading
import json
import time
import random
from .receiver import Receiver
from .result import PowerFlowResult, EMTResult, Result
from .storage import Storage
from ..utils import request

RECEIVER = {
    'default': Receiver,
}
RESULT_DB = {
    'job-definition/cloudpss/emtp': EMTResult,
    'job-definition/cloudpss/sfemt': EMTResult,
    'job-definition/cloudpss/power-flow': PowerFlowResult
}


class Runner(object):
    def __init__(self, taskId, name, job, config, revision, projectRid,
                 **kwargs):

        self.taskId = taskId
        self.db = Storage(taskId, name, job, config, revision, projectRid)
        result = RESULT_DB.get(job['rid'], EMTResult)
        self.result = result(self.db)
        self.receiver = None

    def __listenStatus(self):
        if self.receiver.status() == -1:
            raise Exception(self.receiver.error)
        return self.receiver.isOpen

    def status(self):
        """
        运行状态
        :return: 运行状态  0/1/-1 1 表示正常结束，0 表示运行中， -1 表示数据接收异常


        >>>> runner.status()  
        """
        if self.receiver is None:
            raise Exception('not find receiver')
        return self.receiver.status()

    def __listen(self, **kwargs):

        receiver = kwargs.get('RECEIVER', 'default')
        if type(receiver) is str:
            if receiver not in RECEIVER:
                receiver = RECEIVER['default']
            else:
                receiver = RECEIVER[receiver]
        self.receiver = receiver(self.taskId, self.db, **kwargs)
        self.receiver.connect()

    @staticmethod
    def create(revisionHash, job, config, name=None, rid='', **kwargs):
        '''
            创建一个运行任务

            :params: revision 项目版本号
            :params: job 调用仿真时使用的计算方案，为空时使用项目的第一个计算方案
            :params: config 调用仿真时使用的参数方案，为空时使用项目的第一个参数方案
            :params: name 任务名称，为空时使用项目的参数方案名称和计算方案名称
            :params: rid 项目rid，可为空

            :return: 返回一个运行实例

            >>> runner = Runner.runRevision(revision,job,config,'')
        '''
        taskId = str(int(time.time() * random.random()))
        runner = Runner(taskId, name, job, config, revisionHash, rid, **kwargs)
        event = threading.Event()

        thread = threading.Thread(target=runner.__listen, kwargs=kwargs)
        thread.setDaemon(True)
        thread.start()
        payload = {
            'args': config['args'],
            'job': job,
        }
        while not runner.__listenStatus():
            time.sleep(1)
        r = request('POST',
                    'api/simulation/runner/' + revisionHash + '/' +
                    str(taskId),
                    data=json.dumps(payload))

        return runner





from .matlab import loadPyData
from .httprequests import request
from .yamlLoader import fileLoad
from .dataEncoder import MatlabDataEncoder,DateTimeEncode
__all__ = ['request','fileLoad','MatlabDataEncoder','DateTimeEncode']



JOB_DEFINITIONS={
    "emtp":{
        "rid": "job-definition/cloudpss/emtp",
        "args": {
            "debug": "0", 
            "n_cpu": "1", 
            "solver": "0", 
            "end_time": "1", 
            "task_cmd": "", 
            "step_time": "0.0001", 
            "begin_time": "0", 
            "task_queue": "", 
            "initial_type": "0", 
            "ramping_time": "0", 
            "load_snapshot": "0", 
            "save_snapshot": "0", 
            "solver_option": "0", 
            "output_channels": [["aa", "1000", "compressed", "", "canvas_0_7"]], 
            "max_initial_time": "1",
            "load_snapshot_name": "", 
            "save_snapshot_name": "snapshot",
            "save_snapshot_time": "1"
        }, 
        "name": "电磁暂态仿真方案 1"
    },
    'sfemt':{
        "name": "移频电磁暂态仿真方案 1",
        "rid": "job-definition/cloudpss/sfemt",
        "args": {
            "begin_time": "0",
            "end_time": "1",
            "step_time": "0.0001",
            "output_channels": [
                [
                    "",
                    "1000",
                    "compressed",
                    "",
                    ""
                ]
            ],
            "solver": "0",
            "shift_freq": "50",
            "numerical_oscillation_suppression": "1",
            "ess": "0",
            "ess_time": "1e-6",
            "initial_type": "0",
            "ramping_time": "0",
            "max_initial_time": "1",
            "save_snapshot": "0",
            "save_snapshot_time": "1",
            "save_snapshot_name": "snapshot",
            "load_snapshot": "0",
            "load_snapshot_name": "",
            "task_queue": "",
            "solver_option": "0",
            "n_cpu": "1",
            "task_cmd": "",
            "debug": "0"
        }
    },
    "powerFlow":{
        "name": "潮流计算方案 1",
        "rid": "job-definition/cloudpss/power-flow",
        "args": {
            "UseBusVoltageAsInit": "1",
            "UseBusAngleAsInit": "1",
            "UseVoltageLimit": "1",
            "UseReactivePowerLimit": "1",
            "SkipPF": "0",
            "MaxIteration": "30"
        }
    }
}


from .component import Component


class DiagramImplement(object):
    """
        拓扑实现
    """

    def __init__(self, diagram: dict = {}):
        """
            初始化
        """
        for k, v in diagram.items():
            if k == 'cells':
                self.__dict__[k] = v
                for key, val in v.items():
                    v[key] = Component(val)
            else:
                self.__dict__[k] = v

    def __getitem__(self, attr):
        return super(DiagramImplement, self).__getattribute__(attr)

    def toJSON(self):
        """
            类对象序列化为 dict
            :return: dict

            >>>> diagram.toJSON()
        """
        cells = {}
        for key, val in self.cells.items():
            cells[key] = val.toJSON()
        diagram = {**self.__dict__, 'cells': cells}
        return diagram

    def getAllComponents(self):
        """
            获取所有元件

            :return: dict<Component>

            >>>> diagram.getAllComponents()
        """

        return self.cells



import jwt
import os
import time

public_key = '''-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDPN91AYRu5++yPvL1H1auWZFTh
L+rH9Aa3rDvChZKtPfVetvBsqf0DF0uraGGnyOzaXHvIVYYNWQYgI6YO8e8U3pOP
+qcUb+U22blkhXNo8x48uQkGrLMWO4Ppi5SMMiCsNXPSfpANpZ9E7301WSJdRQLj
XU0E2qmggJ2AwjRGNwIDAQAB
-----END PUBLIC KEY-----'''


def setToken(token):
    """
        设置 用户申请的 sdk token 

        :params: token token 

        >>> cloudpss.setToken(token)
    """
    result = verifyToken(token)
    os.environ['CLOUDPSS_TOKEN'] = token
    os.environ['USER_NAME'] = result['username']


def verifyToken(token):
    result = jwt.decode(token, public_key, algorithms='RS256')
    if result['exp'] - int(time.time()) < 0:
        raise Exception('token 已过期，请重新申请')
    if result.get('type', '') != 'SDK':
        raise Exception('token 错误，请使用正确的 token')
    return result



# coding=UTF-8
from .verify import setToken
from .runner import Runner, Result, EMTResult, PowerFlowResult
from .project import Project, ProjectRevision, ProjectTopology

from .utils import MatlabDataEncoder, DateTimeEncode
# from .function import * as function
__all__ = [
    'setToken', 'Project', 'ProjectRevision', 'ProjectTopology', 'Runner',
    'Result', 'PowerFlowResult', 'EMTResult', 'MatlabDataEncoder',
    'DateTimeEncode'
]
__version__ = '2.0.1'



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
from cloudpss import __version__
setup(
    name='cloudpss',
    version=__version__,
    keywords=("cloudpss", "cloudpss-sdk"),
    description='cloudpss sdk',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT Licence",
    url='https://www.cloudpss.net',
    author='cloudpss',
    author_email='zhangdaming@cloudpss.net',
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    python_requires='>=3.7',
    install_requires=['cffi==1.14.5',
                      'cryptography==3.4.7',
                      'cycler==0.10.0',
                      'pycparser==2.20',
                      'PyJWT==2.1.0',
                      'numpy==1.21.2',
                      'PyYAML==5.4.1',
                      'requests==2.25.1',
                      'websocket-client==0.58.0',
                      'pytz==2021.1'],
)



# coding=UTF-8

import json
import requests
import os
from collections import OrderedDict


def request(method, uri, baseUrl=None, params={}, token=None, **kwargs):
    if baseUrl == None:
        baseUrl = os.environ.get('CLOUDPSS_API_URL', 'https://cloudpss.net/')
    url = requests.compat.urljoin(baseUrl, uri)
    token = os.environ.get('CLOUDPSS_TOKEN', None)
    if token:
        headers = {
            'Authorization': 'Bearer ' + token,
            'Content-Type': 'application/json; charset=utf-8'
        }
    else:
        raise Exception('token undefined')
    r = requests.request(method, url, params=params, headers=headers, **kwargs)
    try:
        r.raise_for_status()
    except Exception as e:
        raise Exception(r.text)
    return r



import yaml
import io
import gzip
import struct
import base64


class JavascriptSchema():
    def float32Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('f' * (len(data)//4),data))
    def float64Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('d' * (len(data)//8),data))
    def uint8Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('B' * (len(data)//1),data))
    def uint8ClampedArray(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('B' * (len(data)//1),data))
    def uint16Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('H' * (len(data)//2),data))
    def uint32Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('I' * (len(data)//4),data))
    def int8Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('b' * (len(data)//1),data))
    def int16Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('h' * (len(data)//2),data))
    def int32Array(self,node):
        data = base64.b64decode(node.value)
        return list(struct.unpack('i' * (len(data)//4),data)).toList()
    
yaml.add_constructor('tag:yaml.org,2002:js/Float32Array',JavascriptSchema.float32Array)
yaml.add_constructor('tag:yaml.org,2002:js/Float64Array',JavascriptSchema.float64Array)
yaml.add_constructor('tag:yaml.org,2002:js/Uint8Array',JavascriptSchema.uint8Array)
yaml.add_constructor('tag:yaml.org,2002:js/Uint8ClampedArray',JavascriptSchema.uint8ClampedArray)
yaml.add_constructor('tag:yaml.org,2002:js/Uint16Array',JavascriptSchema.uint16Array)
yaml.add_constructor('tag:yaml.org,2002:js/Uint32Array',JavascriptSchema.uint32Array)
yaml.add_constructor('tag:yaml.org,2002:js/Int8Array',JavascriptSchema.int8Array)
yaml.add_constructor('tag:yaml.org,2002:js/Int16Array',JavascriptSchema.int16Array)
yaml.add_constructor('tag:yaml.org,2002:js/Int32Array',JavascriptSchema.int32Array)

def fileLoad(fileName):
    f=open(fileName,'r+',encoding='utf-8')
    t=f.buffer.read(2)
    f.close()
    data=None
    if t==b'\x1f\x8b':
        with gzip.open(fileName, 'rb') as input_file:
            with io.TextIOWrapper(input_file, encoding='utf-8') as dec:
                r=dec.read()
                data=yaml.load(r,Loader=yaml.FullLoader)
    else:
        f=open(fileName,'r+',encoding='utf-8')
        data=yaml.load(f,Loader=yaml.FullLoader)
        f.close()
    return data